@@ -179,6 +179,8 @@ class ScreenletSession (object):
 					os.rmdir(self.path)
 				except:
 					print "Failed to remove session dir '%s' - not empty?" % self.name
+				# will delete the last screenlet autostart launcher without needing to use manager
+				utils.delete_autostarter(sl.get_short_name())
 				# ...
 				# quit gtk on closing screenlet
 				sl.quit_on_close = True
@@ -251,7 +253,7 @@ class ScreenletSession (object):
 		else:
 			# create new first instance
 			print 'No instance(s) found in session-path, creating new one.'
-			sl = self.screenlet(session=self, id=self.__get_next_id())
+			sl = self.screenlet(session=self, id=self.__get_next_id(), show_window=False)
 			if sl:
 				# add screenlet to session
 				self.instances.append(sl)
@@ -420,13 +422,18 @@ class ScreenletSession (object):
 		# get options for SL from backend
 		opts = self.backend.load_instance(screenlet.id)
 		if opts:
+			#theme must be read in first so the user's options are not overridden
+			for o in opts:
+				opt = screenlet.get_option_by_name(o)
+				if opt and opt.name=="theme_name":
+					setattr(screenlet, opt.name, opt.on_import(opts[o]))
 			for o in opts:
 				# get the attribute's Option-object from Screenlet
 				opt = screenlet.get_option_by_name(o)
 				# NOTE: set attribute in Screenlet by calling the
 				# on_import-function for the Option (to import
 				# the value as the required type)
-				if opt:
+				if opt and opt.name!="theme_name":
 					setattr(screenlet, opt.name, opt.on_import(opts[o]))
 		# re-enable updates and call redraw/reshape
 		screenlet.disable_updates = False
