@@ -13,7 +13,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# Screenlets Manager - (c) RYX (Rico Pfaus) 2007  / Whise (Helder Fraga) <helder.fraga@hotmail.com>
+# Screenlets Manager - (c) RYX (Rico Pfaus) 2007  / Whise (Helder Fraga) <helder.fraga@hotmail.com>, rastkokaradzic (Rastko Karadzic) <rastkokaradzic@gmail.com>
 #
 # + INFO:
 # This is a graphical manager application that simplifies managing and
@@ -21,12 +21,13 @@
 # per-user Screenlets.
 #
 # + TODO:
+# - Implement filter for screenlet-items, appending them every time all over again is too slow
 # - support for using sessions (via commandline), (NOTE: if a session is selected
 #   all autostart-files need to be removed and created new)
 # - Help-button
 # - menu
 #    - different View-modes (icons/details)
-#
+#	
 
 import pygtk
 pygtk.require('2.0')
@@ -66,6 +67,9 @@ else:
 	DIR_USER		= screenlets.DIR_USER
 	DIR_AUTOSTART = utils.get_autostart_dir()
 
+CATEGORIES_ALL = _("All categories")
+CATEGORIES_MISC = _("Miscellaneous")
+
 # classes
 
 
@@ -84,10 +88,22 @@ class ScreenletsManager(object):
 	daemon_iface = None
 	
 	def __init__ (self):
+		#dict of categories that will be shown in combobox
+		self.available_categories = [CATEGORIES_ALL]
+		
 		# create ui and populate it
 		self.create_ui()
 		# populate UI
 		self.load_screenlets()
+		
+		tcats = sorted(self.tmp_cats.items(),key=lambda(k,v):(v,k))[::-1]
+		i = 0
+		while (i<min(10,len(tcats)-1)):
+			self.available_categories.append(_(tcats[i][0]))
+			i=i+1
+		for cat in self.available_categories:
+			self.combo1.append_text(cat)
+		self.combo1.set_active(0)
 		# if we are running as root, show error
 		if USER == 0:
 			screenlets.show_error(None, _("""Important! You are running this application as root user, almost all functionality is disabled. You can use this to install screenlets into the system-wide path."""), 
@@ -99,10 +115,6 @@ class ScreenletsManager(object):
 			self.connect_daemon()	
 		
 	# screenlets stuff
-
-
-	
-
 	
 	def lookup_daemon (self):
 		"""Find the screenlets-daemon or try to launch it. Initializes 
@@ -145,9 +157,9 @@ class ScreenletsManager(object):
 		"""Delete the selected screenlet from the user's screenlet dir."""
 		sel = self.view.get_selected_items()
 		if sel and len(sel) > 0 and len(sel[0]) > 0:
-			it = self.model.get_iter(sel[0][0])
+			it = self.model_filter.get_iter(sel[0][0])
 			if it:
-				info = self.model.get_value(it, 2)
+				info = self.model_filter.get_value(it, 2)
 				if info and not info.system:
 					# delete the file
 					if screenlets.show_question(None, _('Do you really want to permanently uninstall and delete the %sScreenlet from your system?') % info.name, _('Delete Screenlet')):
@@ -161,37 +173,25 @@ class ScreenletsManager(object):
 				else:
 					screenlets.show_error(None, _('Can\'t delete system-wide screenlets.'))
 		return False
-	
+
 	def load_screenlets (self):
 		"""Load all available screenlets, create ScreenletInfo-objects for
-		them and add them to the iconview-model."""
+		them and add them into global dictionary used later for filtering."""
 		# fallback icon
-		noimg = gtk.gdk.pixbuf_new_from_file_at_size(\
+		self.noimg = gtk.gdk.pixbuf_new_from_file_at_size(\
 			screenlets.INSTALL_PREFIX + '/share/screenlets-manager/noimage.svg', 
 			56, 56)
 		# get list of available/running screenlets
+		self.loaded_screenlets = {}
+		self.tmp_cats = {}
 		lst_a = utils.list_available_screenlets()
-		lst_r = utils.list_running_screenlets()
-
-		if not lst_r:
-			lst_r = []
 		lst_a.sort()
-		lst_filtered = []
-		filter_input = str(self.txtsearch.get_text()).lower()
-		combo_sel = self.combo.get_active()
-		if filter_input != '':
-			for s in lst_a:
-				filter_slname = str(s).lower()
-				filter_find = filter_slname.find(filter_input)
-				if filter_input == None or filter_find != -1:
-					lst_filtered.append(s)
-		if lst_filtered == [] and filter_input == '': lst_filtered = lst_a
-		for s in lst_filtered:
+		for s in lst_a:
 			try:
 				img = utils.get_screenlet_icon(s, 56, 56)
 			except Exception, ex:
 				#print "Exception while loading icon '%s': %s" % (path, ex)
-				img = noimg
+				img = self.noimg
 			# get metadata and create ScreenletInfo-object from it
 			meta = utils.get_screenlet_metadata(s)
 			if meta:
@@ -207,38 +207,28 @@ class ScreenletsManager(object):
 				name	= setfield('name', '')
 				info	= setfield('info', '')
 				author	= setfield('author', '')
+				category= setfield('category', CATEGORIES_MISC)
 				version	= setfield('version', '')
+				#Appends category into dict of categories that counts how much screenlets have defined that category.
+				#It's used later to load top defined categories in category filter combobox 
+				if self.tmp_cats.has_key(category):
+					self.tmp_cats[category] += 1
+				else:
+					self.tmp_cats[category] = 1
+
+				
 				# get info
-				slinfo = utils.ScreenletInfo(s, name, info, author, version, img)
-				# check if already running
-				if lst_r.count(s + 'Screenlet'):
-					slinfo.active = True
-				# check if system-wide
-				#if path.startswith(screenlets.INSTALL_PREFIX):
-				#	print "SYSTEM: %s" % s
-				#	info.system = True
+				slinfo = utils.ScreenletInfo(s, name, info, author,category, version, img)
 			else:
 				print 'Error while loading screenlets metadata for "%s".' % s
-				slinfo = utils.ScreenletInfo(s, '', '', '', '', img)
-			# add to model
-
-
-			if combo_sel == 0:
-				self.model.append(['<span size="9000">%s</span>' % s, img, slinfo])	
-			elif combo_sel == 1:
-				if slinfo.active :self.model.append(['<span size="9000">%s</span>' % s, img, slinfo])	
-			elif combo_sel == 2:
-				if slinfo.autostart == True :self.model.append(['<span size="9000">%s</span>' % s, img, slinfo])	
-			elif combo_sel == 3:
-				if slinfo.system == True :self.model.append(['<span size="9000">%s</span>' % s, img, slinfo])	
-			elif combo_sel == 4:
-				if slinfo.system == False:self.model.append(['<span size="9000">%s</span>' % s, img, slinfo])				
-
-
+				slinfo = utils.ScreenletInfo(s, '','', '','', '', img)
+			#Append screenlet info into global dictionary used later to filter screenlets
+			self.loaded_screenlets[slinfo.name] = slinfo
+			self.model.append(['<span size="9000">%s</span>' % s, img, slinfo])	
 	
 	def get_Info_by_name (self, name):
 		"""Returns a ScreenletInfo-object for the screenlet with given name."""
-		for row in self.model:
+		for row in self.model_filter:
 			if row[2] and row[2].name == name:
 				return row[2]
 		return None
@@ -248,17 +238,17 @@ class ScreenletsManager(object):
 		int the IconView."""
 		sel = self.view.get_selected_items()
 		if sel and len(sel)>0 and len(sel[0])>0:
-			it = self.model.get_iter(sel[0][0])
+			it = self.model_filter.get_iter(sel[0][0])
 			if it:
-				return self.model.get_value(it, 2)
+				return self.model_filter.get_value(it, 2)
 		return None
 	
 	def reset_selected_screenlet(self):
 		sel = self.view.get_selected_items()
 		if sel and len(sel) > 0 and len(sel[0]) > 0:
-			it = self.model.get_iter(sel[0][0])
+			it = self.model_filter.get_iter(sel[0][0])
 			if it:
-				info = self.model.get_value(it, 2)
+				info = self.model_filter.get_value(it, 2)
 				if screenlets.show_question(None, _('Do you really want to reset the %sScreenlet configuration?') % info.name, _('Reset Screenlet')):
 					# delete screenlet's config directory 
 					os.system('rm -rf %s/%s' % (screenlets.DIR_CONFIG, info.name))
@@ -272,7 +262,7 @@ class ScreenletsManager(object):
 	
 	def set_screenlet_active (self, name, active):
 		"""Set the screenlet's active-state to active (True/False)."""
-		for row in self.model:
+		for row in self.model_filter:
 			if row[2].name == name[:-9]:
 				row[2].active = active
 				# if selected, also toggle checkbox
@@ -280,7 +270,67 @@ class ScreenletsManager(object):
 				if sel and sel.name == name[:-9]:
 					self.recreate_infobox(sel)
 				break
-	
+	def visible_cb(self, model, iter, data):
+        	slinfo = model.get_value(iter,2)
+		if(slinfo != None):
+			#Check if filter box value matches screenlet name
+			if self.filter_input != '':
+				filter_slname = str(slinfo.name).lower()
+				filter_find = filter_slname.find(self.filter_input)
+				if filter_find == -1:
+					return False
+			
+			wshow = True
+			#Check if screenlet is currently running
+			if self.combo_type_sel == 1: 
+				if self.loaded_screenlets[slinfo.name].active == False:
+					return False
+			elif self.combo_type_sel == 2:
+				if slinfo.autostart != True:	
+					return False
+			elif self.combo_type_sel == 3:
+				if slinfo.system != True:
+					return False
+			elif self.combo_type_sel == 4:
+				if slinfo.system != False:
+					return False
+			#Check if category selection matches screenlet category
+			if self.available_categories[self.combo_cat_sel]==CATEGORIES_ALL:
+				wshow = True
+			elif self.available_categories[self.combo_cat_sel]==slinfo.category:
+				wshow = True
+			elif (not(slinfo.category in self.available_categories)) and self.available_categories[self.combo_cat_sel]==CATEGORIES_MISC:
+				wshow = True
+			else:
+				wshow = False
+			return wshow
+
+
+
+
+		else:
+			return False
+
+
+
+	def refilter_screenlets(self,widget,id):
+		if id == 'search_clean':
+			self.txtsearch.set_text('')
+			self.filter_input = ''
+		elif id == 'search_changed':
+			self.filter_input = str(self.txtsearch.get_text()).lower()
+		elif id == 'type_changed':
+			self.combo_type_sel = self.combo.get_active()
+			if(self.combo_type_sel == 1):
+				lst_r = utils.list_running_screenlets()
+				if lst_r != None:
+					for item in lst_r:
+						if item[:-9] in self.loaded_screenlets.keys():
+							self.loaded_screenlets[item[:-9]].active=True
+						
+		elif id == 'category_changed':
+			self.combo_cat_sel = self.combo1.get_active()
+		self.model_filter.refilter()
 	# ui creation
 	
 	def create_ui (self):
@@ -311,10 +361,11 @@ class ScreenletsManager(object):
 		vbox.pack_start(hbox, True, True)
 		hbox.show()
 		# iconview
-		self.model= gtk.ListStore(object)
 		self.view = iv = gtk.IconView()
 		self.model = gtk.ListStore(str, gtk.gdk.Pixbuf, object)
-		iv.set_model(self.model)
+		self.model_filter = self.model.filter_new()
+		self.model_filter.set_visible_func(self.visible_cb, None)
+		iv.set_model(self.model_filter)
 		iv.set_markup_column(0)
 		iv.set_pixbuf_column(1)
 		# disable UI for root user
@@ -331,7 +382,7 @@ class ScreenletsManager(object):
 		iv.connect("drag_data_received", self.drag_data_received)
 		# wrap iconview in scrollwin
 		sw = self.slwindow = gtk.ScrolledWindow()
-		sw.set_size_request(560, 350)
+		sw.set_size_request(560, 380)
 		sw.set_shadow_type(gtk.SHADOW_IN)
 		sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
 		sw.add(iv)
@@ -423,27 +474,39 @@ class ScreenletsManager(object):
 		self.label.set_width_chars(70)
 		self.label.set_alignment(0, 0)
 		self.label.set_size_request(-1, 65)
+		self.filter_input=''
     		self.btnsearch = gtk.Button()
+		self.filterbox = gtk.VBox()
     		self.searchbox = gtk.HBox()
     		self.txtsearch = gtk.Entry()
 		self.btnsearch.set_image(gtk.image_new_from_stock(gtk.STOCK_CLOSE, 
 			gtk.ICON_SIZE_BUTTON))
-    		self.btnsearch.connect("clicked",self.redraw_screenlets, 'clean')
-    		self.txtsearch.connect("changed",self.redraw_screenlets, 'enter')
-    		self.txtsearch.connect("backspace",self.redraw_screenlets, 'backspace')
+    		self.btnsearch.connect("clicked",self.refilter_screenlets, 'search_clean')
+    		self.txtsearch.connect("changed",self.refilter_screenlets, 'search_changed')
+    		
 
     		self.searchbox.pack_start(self.txtsearch, 1)
     		self.searchbox.pack_start(self.btnsearch, False)
-		butbox.pack_start(self.searchbox, False,0,3)
+		self.filterbox.pack_start(self.searchbox, False)
+
 		self.combo = gtk.combo_box_new_text()
 		self.combo.append_text(_('All Screenlets'))
 		self.combo.append_text(_('Running Screenlets'))
 		self.combo.append_text(_('Autostart Screenlets'))
 		self.combo.append_text(_('Only native Screenlets'))
 		self.combo.append_text(_('Only third party'))
+		self.combo_type_sel=0
 		self.combo.set_active(0)
-    		self.combo.connect("changed",self.redraw_screenlets, 'enter')
+    		self.combo.connect("changed",self.refilter_screenlets, 'type_changed')
 		self.combo.show()
+		self.combo1 = gtk.combo_box_new_text()
+		self.combo_cat_sel=0
+		self.combo1.set_active(0)
+		self.combo1.connect("changed",self.refilter_screenlets, 'category_changed')
+		self.combo1.show()
+		self.filterbox.pack_start(self.combo1, False)
+		self.filterbox.pack_start(self.combo, False)
+		butbox.pack_start(self.filterbox, False,0,3)
 		butbox.pack_start(but1, False)
 		butbox.pack_start(but2, False)
 		butbox.pack_start(but3, False)
@@ -455,7 +518,6 @@ class ScreenletsManager(object):
 		#sep2 =   gtk.HSeparator()
 		#butbox.pack_start(sep2, False,False,5)
 		butbox.pack_start(but8, False)
-		butbox.pack_start(self.combo, False)
 		#butbox.pack_start(self.label, False)
 		butbox.show_all()
 		hbox.pack_start(butbox, False, False, 10)
@@ -575,10 +637,7 @@ class ScreenletsManager(object):
 		self.bbox.pack_start(ibox, False,False)
 
 	def redraw_screenlets(self,widget,id):
-		if id == 'backspace':
-			if len(self.txtsearch.get_text()) == 1:
-				self.txtsearch.set_text('')
-		elif id == 'clean':
+		if id == 'clean':
 			self.txtsearch.set_text('')
 		else:
 			self.model.clear()
@@ -895,6 +954,7 @@ class ScreenletsManager(object):
 					os.system('sh '+ DIR_AUTOSTART + s + ' &')	
 		elif id == 'closeall':
 			utils.quit_all_screenlets()
+			self.refilter_screenlets(None,'type_changed')
 
 		elif id == 'download':
 			if screenlets.UBUNTU:
@@ -1257,6 +1317,7 @@ class ScreenletsManager(object):
 					widget.set_sensitive(False)
 			else:
 				utils.delete_autostarter(info.name)
+				self.refilter_screenlets(None,'None')
 
 	def toggle_tray (self, widget):
 		"""Callback for handling changes to the tray-CheckButton."""
